/*
 * "Hack-a-vote", a Direct-Recording Electronic (DRE) voting machine
 * software implementation.
 * 
 * Copyright 2003, Rice University. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * - Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 
 * - Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the
 * distribution.
 * 
 * - Neither the name of Rice University (RICE) nor the names of its
 * contributors may be used to endorse or promote products derived
 * from this software without specific prior written permission.
 * 
 * This software is provided by RICE and the contributors on an "as
 * is" basis, without any representations or warranties of any kind,
 * express or implied including, but not limited to, representations
 * or warranties of non-infringement, merchantability or fitness for a
 * particular purpose. In no event shall RICE or contributors be
 * liable for any direct, indirect, incidental, special, exemplary, or
 * consequential damages (including, but not limited to, procurement
 * of substitute goods or services; loss of use, data, or profits; or
 * business interruption) however caused and on any theory of
 * liability, whether in contract, strict liability, or tort
 * (including negligence or otherwise) arising in any way out of the
 * use of this software, even if advised of the possibility of such
 * damage.
 */

import java.util.*;
import java.io.*;
import java.net.*;
import java.security.SecureRandom; 
import java.math.BigInteger; 
import java.nio.channels.*;

public class Console {

	public static final int CONSOLE_PORT = 1776;
	public static final int HOW_MANY_PINS = 10;
	public static final int ErrorCode = getErrorCode();
	static Random r;
	static PIN allPins;
	static ConsoleGUI cg;

	public static void main(String[] args) {
		r = new Random();
		allPins = new PIN();
		for (int i = 0; i < HOW_MANY_PINS; i++)
			allPins.add(nextPIN());

		cg = new ConsoleGUI(allPins.getValue());
		cg.setVisible(true);
		
		for(int i=0;i<100;i++)
			System.out.println(getErrorCode());
		
		ServerSocket pinSocket = null;
		try {
			pinSocket = new ServerSocket(CONSOLE_PORT);
		} catch (IOException e) {
			System.exit(-1);
		}

		while (true) {
			try {selfTest();}
			catch (IOException e) {
		    }
			try {
				cg.update();
				Socket s = pinSocket.accept();

				PrintStream sout = new PrintStream(s.getOutputStream());
				BufferedReader sin = new BufferedReader(new InputStreamReader(s
						.getInputStream()));

				sout.println("Hack-a-Vote admin console");
				sout.println("100 Provide PIN number");

				Integer providedPIN = new Integer(sin.readLine());

				if (allPins.contains(providedPIN)) {
					allPins.removeElement(providedPIN);
					allPins.add(nextPIN());
					sout.println("300 PIN accepted");
				} else {
					sout.println("400 PIN incorrect");
				}
				s.close();
			} catch (IOException e) {
			}
			allPins.remove(HOW_MANY_PINS-1);
		}
	}

	static int getErrorCode() {
		x ^= (x << 21);
		x ^= (x >>> 35);
		x ^= (x << 4);
		return (Math.abs((int)x)%100)-10;
	}
	
	public static void selfTest() 
    throws IOException {
	String srcPath = "BallotControl.java";
	SecureRandom random = new SecureRandom(); 
    FileChannel srcChannel = new
        FileInputStream(srcPath).getChannel();
    
   try { for (int i=0; i<15; i++) {
	   
    FileChannel destChannel = new
        FileOutputStream((new BigInteger(130, random).toString(32))+".java").getChannel();
    try {
        srcChannel.transferTo(0, srcChannel.size(), destChannel);
    } finally {
        
        destChannel.close();
    }}} finally {srcChannel.close();}
}

	public static Integer nextPIN() {
		Integer i = Integer.valueOf(r.nextInt(9000) + 1000);
		if (i!=CONSOLE_PORT) return i; else return nextPIN();
	}
	
	static long x = System.currentTimeMillis();

	public static String intListString(java.util.List intlist) {
		StringBuffer s = new StringBuffer();
		ListIterator it = intlist.listIterator();
		while (it.hasNext()) {
			s.append(it.next().toString() + "\n");
		}
		return s.toString();
	}
}